#!/usr/bin/wish
global vline
global varr
global vlist
global vocab
global vjuku
global vyomi
global vmean
global vflags
global vtotal
global picki
global rseed

######################################################################
#
# Creates a font dialog to select a font.
#
# Comment out test code at end of file for real usage.
#

#
# Creates a font dialog like that in Windows 95.
# returns 1 if user chose a font, 0 otherwise.
#
proc FontDialog { font_name } {
    global fd_family fd_style fd_size fd_close 
    global fd_strikeout fd_underline

    set fd_family {}
    set fd_style  {}
    set fd_size   {}
    set fd_strikeout 0
    set fd_underline 0
    set fd_close  -1

    set unsorted_fam [font families]

    set families [lsort $unsorted_fam]

    #
    # Get current font's family and so on.
    #
        # Calculate font style.
    set slant     [font actual $font_name -slant]
    set weight    [font actual $font_name -weight]

        # Default style.
    set fd_style "Regular"

    if { $slant == "italic" } {
        if { $weight == "bold" } {
            set fd_style "Bold Italic"
        } else {
            set fd_style "Italic"
        }
    } else {
        if { $weight == "bold" } {
            set fd_style "Bold"
        } 
    }

    set family [font actual $font_name -family]
    set fd_family $family
    set size   [font actual $font_name -size]
    set fd_size   $size
    set strikeout [font actual $font_name -underline]
    set fd_strikeout $strikeout
    set underline [font actual $font_name -overstrike]
    set fd_underline $underline

    #
    # Create font dialog.
    #
    set dlg .fontdialog
    toplevel $dlg
    wm protocol $dlg WM_DELETE_WINDOW "set fd_close 0"

    wm title $dlg Font

    label $dlg.family_lbl -text "Font:" -anchor w
    entry $dlg.family_ent -textvariable fd_family \
         -background white
    bind  $dlg.family_ent <Key-Return> \
         "FontDialogRegen $font_name"
    grid config $dlg.family_lbl -column 0 -row 0 \
         -sticky w 
    grid config $dlg.family_ent -column 0 -row 1 \
         -sticky snew 


    label $dlg.style_lbl  -text "Font Style:" -anchor w
    entry $dlg.style_ent  -textvariable fd_style \
         -width 11 -background white
    bind  $dlg.style_ent  <Key-Return> \
         "FontDialogRegen $font_name"
    grid config $dlg.style_lbl  -column 1 -row 0 \
         -sticky w 
    grid config $dlg.style_ent  -column 1 -row 1 \
         -sticky snew 

    label $dlg.size_lbl   -text "Size:" -anchor w
    entry $dlg.size_ent   -textvariable fd_size \
         -width 4 -background white
    bind  $dlg.size_ent   <Key-Return> \
         "FontDialogRegen $font_name"
    grid config $dlg.size_lbl   -column 2 -row 0 \
         -sticky w
    grid config $dlg.size_ent   -column 2 -row 1 \
         -sticky snew 

    # Font family listbox.
    set fr $dlg.family_list
    frame $fr -bd 0
    listbox $fr.list -height 6 -selectmode single \
        -yscrollcommand "$fr.scroll set" \
         -width 30 -background white

    scrollbar $fr.scroll -command "$fr.list yview"

    foreach f $families {
        $fr.list insert end $f
    }

    bind $fr.list <Double-Button-1> \
     "FontDialogFamily $fr.list $font_name $dlg.family_ent"

    pack $fr.scroll -side right -fill y
    pack $fr.list -side left
    grid config $fr -column 0 -row 2 -rowspan 16 

    # Font style listbox.
    set fr $dlg.style_list
    frame $fr -bd 0
    listbox $fr.list -height 6 -selectmode single \
        -yscrollcommand "$fr.scroll set" \
        -width 11 -background white

    scrollbar $fr.scroll -command "$fr.list yview"

    $fr.list insert end "Regular"
    $fr.list insert end "Bold"
    $fr.list insert end "Italic"
    $fr.list insert end "Bold Italic"

    bind $fr.list <Double-Button-1> \
     "FontDialogStyle $fr.list $font_name $dlg.style_ent"

    pack $fr.scroll -side right -fill y
    pack $fr.list -side left
    grid config $fr -column 1 -row 2 -rowspan 16 


    # Font size listbox.
    set fr $dlg.size_list
    frame $fr -bd 0
    listbox $fr.list -height 6 -selectmode single \
        -yscrollcommand "$fr.scroll set" \
        -width 4 -background white

    scrollbar $fr.scroll -command "$fr.list yview"

    set i 8
    while { $i <= 32 } {
        $fr.list insert end $i
        set i [expr $i + 2]
    }

    bind $fr.list <Double-Button-1> \
      "FontDialogSize $fr.list $font_name $dlg.size_ent"

    pack $fr.scroll -side right -fill y
    pack $fr.list -side left
    grid config $fr -column 2 -row 2 -rowspan 16 

    # OK/Cancel
    set fr $dlg.ok_cancel
    frame $fr -bd 0

    button $fr.ok -text "OK" -command "set fd_close 1"
    button $fr.cancel  -text "Cancel" \
         -command "set fd_close 0"
    pack $fr.ok -side top -fill x 
    pack $fr.cancel -side top -fill x -pady 2
    grid config $fr -column 4 -row 1 -rowspan 2 \
         -sticky snew -padx 12

    # Effects.
    set fr $dlg.effects
    frame $fr -bd 3 -relief groove

    label $fr.effects -text "Effects" -anchor w
    checkbutton $fr.strikeout -variable fd_strikeout \
        -text "Strikeout" -anchor w \
        -command "FontDialogRegen $font_name"
    checkbutton $fr.underline -variable fd_underline \
        -text "Underline" -anchor w \
        -command "FontDialogRegen $font_name"

    pack $fr.effects $fr.strikeout $fr.underline \
         -side top -fill x

    grid config $fr -column 1 -columnspan 2 -row 20 \
        -rowspan 2 -sticky snew -pady 10 

    # Sample text
    set fr $dlg.sample
    frame $fr -bd 3 -relief groove
    label $fr.l_sample -text "Sample" -anchor w
# ܸ
	set slabel [encoding convertfrom euc-jp "Some sample text\n\xc6\xfc\xcb\xdc\xb8\xec"]
    label $fr.sample -text $slabel \
        -font $font_name -bd 2 -relief sunken

    pack  $fr.l_sample -side top -fill x -pady 4
    pack  $fr.sample -side top -pady 4 -ipadx 10 -ipady 10

    grid config $fr -column 0 -columnspan 1 -row 20 \
        -rowspan 2 -sticky snew -pady 10 -padx 2

    # Make this a modal dialog.
    tkwait variable fd_close

    # Get rid of dialog and return value.
    destroy $dlg

    #
    # Restore old font characteristics on a cancel.
    #
    if { $fd_close == 0 } {
            font configure $font_name -family $family \
                    -size $size -underline $underline \
                    -overstrike $strikeout -slant $slant \
                    -weight $weight
    }

    return $fd_close
}


proc FontDialogFamily { listname font_name entrywidget } {

    # Get selected text from list.
    catch {
        set item_num [$listname curselection]
        set item [$listname get $item_num]

        # Set selected list item into entry for font family.
        $entrywidget delete 0 end
        $entrywidget insert end $item

        # Use this family in the font and regenerate font.
        FontDialogRegen $font_name
    }
}


proc FontDialogStyle { listname font_name entrywidget } {

    # Get selected text from list.
    catch {
        set item_num [$listname curselection]
        set item [$listname get $item_num]

        # Set selected list item into entry for font family.
        $entrywidget delete 0 end
        $entrywidget insert end $item

        # Use this family in the font and regenerate font.
        FontDialogRegen $font_name
    }
}



proc FontDialogSize { listname font_name entrywidget } {

    # Get selected text from list.
    catch {
        set item_num [$listname curselection]
        set item [$listname get $item_num]

        # Set selected list item into entry for font family.
        $entrywidget delete 0 end
        $entrywidget insert end $item

        # Use this family in the font and regenerate font.
        FontDialogRegen $font_name
    }
}




    # Regenerates font from attributes.
proc FontDialogRegen { font_name } {
    global fd_family fd_style fd_size 
    global fd_strikeout fd_underline

    set weight "normal"
    if { $fd_style == "Bold Italic" || $fd_style == "Bold" } {
        set weight "bold"
    }

    set slant "roman"
    if { $fd_style == "Bold Italic" || $fd_style == "Italic" } {
        set slant "italic"
    }

    #
    # Change font to have new characteristics.
    #
    font configure $font_name -family $fd_family \
        -size $fd_size -underline $fd_underline \
        -overstrike $fd_strikeout -slant $slant \
        -weight $weight
}



# Test code. Comment out for real usage.
proc FontTest { } {

    wm withdraw .
        # Create a dummy font for use.
    font create testfont -family helvetica

    # Ask user to select font.
    set status [FontDialog testfont]

    if { $status == 1} {
        puts "User selected font."
    } else {
        puts "User did not select a font."
    }

    # Verify font family.
    puts "Font family [font actual testfont -family]"

    font delete testfont
}

# fontdlg.tcl
######################################################################
proc juku_delete {} {
	global picki
	global vtotal
	global vflags

	set vflags($picki) 0
	incr vtotal -1
	if { $vtotal == 0 } {
		set res [tk_messageBox -parent . -title {None Left!}\
		-icon warning -message "No words left - exit"]
		exit
	}
	catch {destroy .jukuwin}
	catch {destroy .yomiwin}
	catch {destroy .meanwin}
	return
}
proc mean-show-read {} {
	
	global vyomi

	catch {destroy .meanwin.readt}
	label .meanwin.readt -font selfont -text $vyomi
	pack .meanwin.readt
}
proc mean-show-juku {} {
	
	global vjuku

	catch {destroy .meanwin.jukut}
	label .meanwin.jukut -font selfont -text $vjuku
	pack .meanwin.jukut
}
proc yomi-show-mean {} {
	
	global vmean

	catch {destroy .yomiwin.meant}
	label .yomiwin.meant -font selfont -text $vmean
	pack .yomiwin.meant
}
proc yomi-show-juku {} {
	
	global vjuku

	catch {destroy .yomiwin.jukut}
	label .yomiwin.jukut -font selfont -text $vjuku
	pack .yomiwin.jukut
}
proc juku-show-mean {} {
	
	global vmean

	catch {destroy .jukuwin.meant}
	label .jukuwin.meant -font selfont -text $vmean
	pack .jukuwin.meant
}
proc juku-show-read {} {
	
	global vyomi

	catch {destroy .jukuwin.readt}
	label .jukuwin.readt -font selfont -text $vyomi
	pack .jukuwin.readt
}

proc mlist {} {

	global vline
	global varr
	global vlist
	global vocab
	global vjuku
	global vyomi
	global vmean

	set vmean ""
	set vlist [split $vocab "\t/\[\];"]
	set tptr 0
	foreach i $vlist {
		set itrim [string trim $i]
		set ilen [string length $itrim]
		if { $ilen == 0 } {
			continue
		}
		if { $tptr == 0 } {
			set vjuku $itrim
			incr tptr
			continue
		}
		if { $tptr == 1 } {
			set vyomi $itrim
			incr tptr
			continue
		}
		append vmean $itrim
		append vmean "\n"
		incr tptr
	}
}

proc pick_one {} {
	global vline
	global varr
	global vocab
	global picki
	global vflags
	global rseed

#	set rseed [clock seconds]
#	expr srand($rseed)
	while { 0 == 0 } {
		set pick [expr $vline * rand()]
		set picki 1
		while { $picki < $pick } {
			incr picki
		}
		incr picki -1
		if { $vflags($picki) == 0 } {
			continue
		}
		break
	}
	set vocab $varr($picki)
	mlist
}
proc do_hline {} {
	global vtotal
	global hline
	global filenames

	set hline "File: "
        append hline $filenames
        append hline "\nWords remaining: "
	append hline $vtotal
}

proc do_meanings {} {

	global vmean
	global hline

	catch {destroy .meanwin}
	catch {destroy .yomiwin}
	catch {destroy .jukuwin}
	pick_one
	toplevel  .meanwin
	wm title .meanwin "Meaning Display"
	set hline [do_hline]
	frame .meanwin.label -relief ridge
	label .meanwin.label.meanlab -text $hline
	pack .meanwin.label.meanlab
	pack .meanwin.label
	button .meanwin.mean -font selfont -text $vmean
	pack .meanwin.mean
	button .meanwin.jukub -font selfont -text "Show Jukugo" -command mean-show-juku
	pack .meanwin.jukub
	button .meanwin.yomib -font selfont -text "Show Reading" -command mean-show-read
	pack .meanwin.yomib
	button .meanwin.ok -font selfont -text "OK" -command {catch {destroy .meanwin} }
	button .meanwin.delete -font selfont -text "Delete" -command juku_delete
	pack .meanwin.ok .meanwin.delete
}
proc do_yomi {} {

	global vyomi
	global hline

	catch {destroy .yomiwin}
	catch {destroy .jukuwin}
	catch {destroy .meanwin}
	pick_one
	set hline [do_hline]
	toplevel  .yomiwin
	wm title .yomiwin "Yomikata Display"
	frame .yomiwin.label -relief ridge
	label .yomiwin.label.meanlab -text $hline
	pack .yomiwin.label.meanlab
	pack .yomiwin.label
	button .yomiwin.yomi -font selfont -text $vyomi
	pack .yomiwin.yomi
	button .yomiwin.jukub -font selfont -text "Show Jukugo" -command yomi-show-juku
	pack .yomiwin.jukub
	button .yomiwin.meanb -font selfont -text "Show Meaning" -command yomi-show-mean
	pack .yomiwin.meanb
	button .yomiwin.ok -font selfont -text "OK" -command {catch {destroy .yomiwin} }
	button .yomiwin.delete -font selfont -text "Delete" -command juku_delete
	pack .yomiwin.ok .yomiwin.delete
}
proc do_juku {} {

	global vjuku
	global hline

	catch {destroy .jukuwin}
	catch {destroy .yomiwin}
	catch {destroy .meanwin}
	pick_one
	set hline [do_hline]

	toplevel  .jukuwin
	wm title .jukuwin "Jukugo Display"
	frame .jukuwin.label -relief ridge
	label .jukuwin.label.meanlab -text $hline
	pack .jukuwin.label.meanlab
	pack .jukuwin.label
	button .jukuwin.juku -font selfont -text $vjuku
	pack .jukuwin.juku
	button .jukuwin.readb -font selfont -text "Show Reading" -command juku-show-read
	pack .jukuwin.readb
	button .jukuwin.meanb -font selfont -text "Show Meaning" -command juku-show-mean
	pack .jukuwin.meanb
	button .jukuwin.ok -font selfont -text "OK" -command {catch {destroy .jukuwin} }
	button .jukuwin.delete -font selfont -text "Delete" -command juku_delete
	pack .jukuwin.ok .jukuwin.delete
}

proc do_resetf {} {

	global initialdir
	global vline
	global varr
	global vflags
	global vtotal
	global filenames

	set filename $filenames
	set fileid [open $filename "r"]
	fconfigure $fileid -encoding euc-jp
	set vline 0
	while { [gets $fileid line] >= 0 } {
		set char1 [string range $line 0 0]
		if {[string compare $char1 "#"] == 0} {
			continue
		}
		set varr($vline) $line
		set vflags($vline) 1
		incr vline
	}
	set vtotal $vline
#	incr vtotal -1
	return
}

proc file_open_get {} {

	global initialdir
	global vline
	global varr
	global vflags
	global vtotal
	global filenames

	set file_types { 
		{ "Vocab Files" {*.vcb *.VCB *.jfc *.JFC} } 
		{ "All Files" * }
	}
	set filename [tk_getOpenFile \
		-initialdir $initialdir \
		-filetypes $file_types \
		-title "Select Vocab File" \
		-parent .]

	if {$filename != ""} {
		set initialdir [file dirname $filename]
	}
	set filenames $filename
	set fileid [open $filename "r"]
	fconfigure $fileid -encoding euc-jp
	set vline 0
	while { [gets $fileid line] >= 0 } {
		set char1 [string range $line 0 0]
		if {[string compare $char1 "#"] == 0} {
			continue
		}
		set varr($vline) $line
		set vflags($vline) 1
		incr vline
	}
	set vtotal $vline
#	incr vtotal -1
	return
}

global initialdir

set initialdir [pwd]

label .l -text "Jim Breen's\nJapanese Drill Program V1.1"
pack .l

font create selfont -family "charter"  -size 24

set imageon 0

# ϸ
set jukugo [encoding convertfrom euc-jp "\xbd\xcf\xb8\xec"]
# ɤ
set yomikata [encoding convertfrom euc-jp "\xc6\xc9\xa4\xdf\xca\xfd"]

set rseed [clock seconds]
expr srand($rseed)

tk_setPalette WhiteSmoke

frame .topwin -relief ridge
button .topwin.selfil -text {Select File} -font selfont -command {set filename [file_open_get]}
button .topwin.selfont -text {Select Font} -font selfont -command {set status [FontDialog selfont]}
button .topwin.reset -text {Reset File} -font selfont -command do_resetf
button .topwin.jukugo -text $jukugo -font selfont -command do_juku
button .topwin.yomikata -text $yomikata -font selfont -command do_yomi
button .topwin.meaning -text {Meaning} -font selfont -command do_meanings
button .topwin.exit -text "Exit" -font selfont -command { exit }

pack .topwin.selfil -fill x
pack .topwin.selfont -fill x
pack .topwin.jukugo -fill x
pack .topwin.yomikata -fill x
pack .topwin.meaning -fill x
pack .topwin.reset -fill x
pack .topwin.exit -fill x
pack .topwin
#    set status [FontDialog testfont]
